% Illustration of the Matlab code for equitable designs
% J. Rendas (CNRS/I3S)

%% generic parameters

d=5;                   % dimension of the input space
Delta =.001;           % size of the grid used to compute numerical derivatives
point = 1.*ones(1,d);  % point at which the derivatives will be computed

%%                                    c=1 (single cross derivative for each pair of input directions)
% available functions:
% function [X ddX] = designc1(d)
%    returns in X a (d,1)-cycle equitable design (that enables computation
%    of all cross derivatives in a space of dimension d.
%   
%    output variable ddX is a matrix 4xC^d_d that describes the 4-cycles that
%    compute the mixed 2nd-order derivatives
%    dX is recursivelly computed: dX(d) = [dX(d-1) c(d,1) ... c(d,d-1)]
%    where c(d,i) is a 4-element column vector indicating the four points that
%    are used to compute the derivative along X_d and X_i. A numerical estimate of the
%    corresponding derivative is obtained as in the example below.
%   In the example, point is the point at which the derivative is computed,
%   and Delta is the grid increment. The function f is provided in the zip
%   file and computes the function x_1x_2^2...x_d^d.
%
% -function n = designc1size(d)
%      computes the size of the equitable (d,1) design
%
% function [X MdX MddX N_design] = designc1withderivatives(d)
%   returns in X a (d,1)-cycle equitable design
% 
%   N_design is design size = 4+\sum_{i=3}^d i
%
%   output variable MdX is a (d+1)*d TIMES N_design
%   derivatives wrt X_i are the (i-1)(d+1)+1 : i(d-1) lines of the product by
%   the evaluation vector
%
%   output variable MddX is a d TIMES N_design matrix. Line i indicates yields
%   the derivative wrt X_i


% explicit computation of the distinct cross derivatives
[X ddX] = designc1(d); 
n=1;
deriv = NaN(d,d);
for i=2:d
    for j=1:i-1
        f_design = f(ones(4,1)*point + X(ddX(:,n),:)*Delta);
        deriv(i,j) = [1 -1 -1 1] * f_design / (Delta^2);
        n=n+1;
    end
end

% size of design
N = designc1size(d);

% computation of both derivatives and cross-derivatives

% generate multiplication matrices for computation of all principal and
% mixed effects
[X MdX MddX N_design] = designc1withderivatives(d);

% compute the function at all design points
f_design = f(ones(N,1)*point + X*Delta);

% compute all local principal effects (1st order derivatives)
D_x = MdX * f_design / (Delta);

% and their averaged values
S = [ones(1,d), zeros(1, d*(d-1))];
for i=2:d
    S = [S; zeros(1,d*(i-1)) ones(1,d) zeros(1, d*(d-i)) ];
end
D_X_av = S*D_x/d;

% compute all mixed effects (2nd order derivatives)
DD_x = MddX * f_design / (Delta^2);

%%                               c=2 (two cross derivative for each pair of input directions)
% function [ X MdX MddX] = designc2(d)
% 
% -- X is a (d,2)-cycle equitable design 
%
% Note: the operator (+) that is used in the paper from eq. (8) onwards 
% -- and that is not properly defined in the paper, we apologize for this omission - 
% is the logical operator or begin zero only i the two
% operands are equal to zero.

% it implements the (d,2)-cycle equitable designs as presented in the paper considering the identity permutation

% computation of the design
[ X_2 MdX MddX] = designc2(d);
% design size
N = size(X_2,1);
% compute f at design points
f_design_2 = f(ones(N,1)*point + X_2*Delta);

% principal effects
df = MdX*f_design_2/Delta;
% mixed effects
ddf = MddX*f_design_2/(Delta^2);


