function  [X ddX] = designc1(d)
% Returns in X a (d,1)-cycle equitable design
% Uses Eqs (4) and (6) of paper by Fedou and Rendas.
%
% output variable ddX is a matrix 4xC^d_d that describes the 4-cycles that
% compute the mixed 2nd-order derivatives
% ddX is recursivelly computed: ddX(d) = [ddX(d-1) c(d,1) ... c(d,d-1)]
% where c(d,i) is a 4-element column vector indicating the four points that
% are used to compute the derivative along X_d and X_i. A numerical estimate of the
%  derivative is obtained by [1 -1 -1 1] f(c(d,i)/(Delta^2) where Delta is
%  the increment between grid points
%
% J. Rendas, JM Fedou, Ocotber 2013
%
if d==1
    display('designc1: input space dimension must be greater than one (aborting)')
    X = []; ddX = [];
    return
end

if d==2
    % design is the entire Q_2
    X = [0 0; 1 0; 0 1; 1 1];
    % only a single cross deriv (x_1,x_2) involving all 4 points.
    ddX = [1; 2; 3; 4];
else
    % recursively build the design
    [X_d_ ddX_] = designc1(d-1);
    n = size(X_d_,1);    % number of points in previous design

    X = [ X_d_ zeros(n,1); 
         [ Line(d-1) ones(d,1) ]]; % [ Line(d-1) ones(d,1) ] is X_d * (polynomial in eq. (6))
    
    % keep track of the new cycles (X_i,X_d), i=1,...,d-1
    m = size(ddX_, 2); % number of distinct cross derivatives X_i X_j, with i != j
    if d>= 4
        point = [ ddX_(4, m-d+2); ddX_(4,end)];
        New = [[ddX_(1:2,m-d+3: m) point]
            [n+1: n+d-1];
            [n+2: n+d]];
        %upper part are the points in X_d_ used to compute the cross
        %derivatives wrt X_d, lower part are the points in the added part
        %that complete the corresponding cycles
    else
        point = [2; 4];
        New = [[[1; 2] point];
            [n+1: n+d-1];
            [n+2: n+d]];
    end
    
    % append new cycles to the returned matrix
    ddX =[ddX_ New];
    
    
end
end

function [L] = Line(d)
% generates the linear design in space of dimension d
% polynomial 1+ x_1 + x_1 x_2 + ... + x_1...x_d

L = zeros(1,d);
for i=1:d
    L = [L; [  ones(1,i) zeros(1,d-i)]];
end
end

