function [ X MdX MddX] = designc2(d)
% Equitable designs (d,2) as in the paper by Fedou & Rendas
% Implements eqs (4) and (8) in the paper
%
% returns 
% -- in X a (d,2)-cycle equitable design 
% -- sigma is the permutation used (the elements that are randomly chosen in the
%    constrution of the design).

% Note: the operator (+) that is used in the paper from eq. (8) onwards 
% -- and that is not properly defined in the paper, we apologize for this omission - 
% is the logical operator exclusive or begin one only if only of the two
% operands are equalt to one and zero otherwise

% J. Rendas, J. M. Fedou, Jan 2017


if (d<3)
    display('designc2: d must be greater than 2 (aborting).')
    X=[]; MdX =[]; MddX=[];
    return
end

% simpler construction (eq. (6)
if d==3
    % initialisation with Q_3
    X = [0 0 0; 
         1 0 0; 
         1 1 0; 
         1 1 1;
         0 1 1;
         0 0 1; 
         1 0 1;
         0 1 0];
     % the matrix that yields the pairs of X-derivatives
    MddX = [1 -1 1 0 0 0 0 -1;
            0 0 0 1 -1 1 -1 0; % the two deriv wrt (x_1,x_2)
            1 -1 0 0 0 -1 1 0;
            0 0 -1 1 -1 0 0 1; % the two deriv wrt (x_1,x_3)
            1 0 0 0 1 -1 0 -1;
            0 1 -1 1 0  0 -1 0]; % the two deriv wrt (x_2,x_3)
    % the matrix that yields the 2(d-1)=4 derivatives wrt each coordinate
    MdX = [-1 1 0 0 0 0 0 0;
            0 0 1 0 0 0 0 -1;
            0 0 0 1 -1 0 0 0;
           0 0 0 0 0 -1 1 0; % x1
           0 -1 1 0 0 0 0 0;
           -1  0 0 0 0 0 0 1;
           0 0 0 1 0 0 -1 0;
           0 0 0 0 1 -1 0 0; % x2
           -1 0 0 0 0 1 0 0;
           0 -1 0 0 0 0 1 0;
           0 0 -1 1 0 0 0 0;
           0 0 0 0 1 0 0 -1]; % x3
           
else
    [X_d_ MdX_ MddX_]  = designc2(d-1);              % design size

    [New dOld dNew ddOld] = LineAndReverse(d);
        
    X = [X_d_ zeros(size(X_d_,1),1);  
        New  ];
    
    % computation of derivatives
    MdX = [];
    n_d_ = 2*(d-2);
    n_d = 2*(d-1);
    % i<d
    N_ = 2*(d-2); % number of repeated derivatives in the previous design
    for i=1:d-1
        MdX = [MdX;
            MdX_(N_*(i-1)+1:N_*i, :) zeros(N_, size(New,1));
            dOld(2*i-1:2*i, :)];
    end
    %     i=d
    MdX = [MdX;
        dNew(end-n_d+1:end,:)];
    
    
    % computation of X-derivatives
    MddX = [MddX_ zeros(size(MddX_,1), size(New,1));
              ddOld ];
end
end

function [L Deriv_old Deriv_new DOld] = LineAndReverse(d)
% generates (in L) the linear design in dimension d
% polynomial 1+ x_1 + x_1 x_2 + ... + x_1...x_d + x2*..*x_d + x3*..*x_d +
% .. .+ x_d
%
% * Deriv_old is the matrix for computation of the new derivatives wrt x_i, i<d
% * Deriv_new is the derivatives wrt x_d
% * DOld is the matrix that enables computation of the d X-derivatives wrt
%   x_dx_i, i < d

L = zeros(1,d);
for i=1:d-1
    L = [L; L(end,:)+Euclides(i,d)];
end
for i=1:d-2
    L = [L; L(end,:)-Euclides(i,d)];
end
L(:,end) = ones(2*d-2,1);


% the matrix for the computation of the new X-derivatives dx_idx_d, i< d
if d == 3
    DOld = [ 1 -1 1 0 0 0 0 -1;
            0 0 0 1 -1 1 -1 0;
            1 -1 0 0 0 1 -1 0;
            0 0 -1 1 -1 0 0 1;
            1 0 0 0 1 -1 0 -1;
            0 1 -1 1 0 0 -1 0];
else  
    DOld = [];
     n_d_1 = designc2size(d-1);
     n_d = designc2size(d);

    for i=1: 2
        DOld = [DOld;
            Euclides(i,n_d) - Euclides(i+1,n_d) - Euclides(n_d_1+i, n_d) +  Euclides(n_d_1+i+1, n_d);
            Euclides(Ones(d-1)+i,n_d) - Euclides(Ones(d-1)+i-1,n_d) + Euclides(Ones(d)+i-1,n_d) - Euclides(Ones(d)+i,n_d)
            ];
    end
    i=i+1;
    while i<= d-1
        if i < d - 2
            DOld = [DOld;
                Euclides(Ones(d-1)+i,n_d) - Euclides(Ones(d-1)+i-1,n_d) + Euclides(Ones(d)+i-1,n_d) - Euclides(Ones(d)+i,n_d); % 
                
                
                Euclides(Ones(i-1) ,n_d) - Euclides(Ones(i),n_d) + Euclides(Ones(d)-(d-i)+1,n_d) - Euclides(Ones(d)-(d-i),n_d); % 
                ];
        elseif i== d-2
            DOld = [DOld;
                Euclides(Ones(i-1),n_d) - Euclides(Ones(i),n_d) + Euclides(Ones(d)-1, n_d) -  Euclides(Ones(d)-2, n_d); % checked d=5,6 
                Euclides(First(d-1),n_d) - Euclides(n_d,n_d) + Euclides(n_d-1,n_d) - Euclides(designc2size(d-1),n_d); % checked d=5, 6, 7
                ];
        elseif i == d-1
            DOld = [DOld;
                Euclides(1,n_d) - Euclides(First(d-1),n_d) + Euclides(n_d, n_d) -  Euclides(First(d), n_d); % checked: d=4, 5, 6, 7
                Euclides(Ones(i-1),n_d) - Euclides(Ones(i),n_d) + Euclides(Ones(d)-(d-i)+1,n_d) - Euclides(Ones(d)-(d-i),n_d); %checked d=6,5, 4
                ];
        end
        i=i+1;
    end
end

n_d = designc2size(d);

% the derivatives wrt x_d
Deriv_new =[];
for i=1:min(d-1,4) % in the direct sense
    Deriv_new = [Deriv_new;
        Euclides(First(d)+i-1,n_d) - Euclides(i,n_d)];
end
for i=5:d-1
    Deriv_new = [Deriv_new;
        Euclides(First(d)+i-1,n_d) - Euclides(Ones(i-1),n_d)];
end
for i=d:2*(d-1)-1
    Deriv_new = [Deriv_new;
        Euclides(Ones(d)+(i-d),n_d) - Euclides(Ones(d-1)+(i-d),n_d)];
end
%last 
Deriv_new = [Deriv_new;
        - Euclides(First(d-1),n_d) + Euclides(n_d,n_d)];        

% the matrix for computation of the new derivatives wrt x_i, i<d
Deriv_old =[];

for i=1:d-2
    a = Euclides(First(d)+i,n_d) - Euclides(First(d)+i-1,n_d);
    b = -Euclides(Ones(d)+i, n_d) + Euclides(Ones(d)+i-1,n_d) ; 
    Deriv_old = [Deriv_old;
        a; b];
end
i =d-1;
a = Euclides(First(d)+i,n_d) - Euclides(First(d)+i-1,n_d);
b = -Euclides(First(d), n_d) + Euclides(n_d,n_d) ;  % the last edge in the cycle
Deriv_old = [Deriv_old;
    a; b];
end



function k = Ones(d)
% gives the number of the design X_d that has all ones
if d==2
    k=3;
elseif d==3
    k=4;
else
    k = designc2size(d-1)+d;
end
end

function e = Euclides(i,d)
% auxiliary function to LinePerm(.)
if i>d 
    display('wrong input i>d in Euclides')
    e=[];
    return
end
e = zeros(1,d);
e(i) = 1;
end


function p = First(d)
% number of the first new design point in X_d
 if d == 3
     p =6;
 else
     p = designc2size(d-1)+1;
 end
end

